@@ -1,6 +1,8 @@
 //! sync git api (various methods)
 
-use super::CommitId;
+use super::{
+	repository::repo, CommitId, RepoPath, ShowUntrackedFilesConfig,
+};
 use crate::{
 	error::{Error, Result},
 	sync::config::untracked_files_config_repo,
@@ -23,54 +25,28 @@ pub struct Head {
 }
 
 ///
-pub fn is_repo(repo_path: &str) -> bool {
+pub fn is_repo(repo_path: &RepoPath) -> bool {
 	Repository::open_ext(
-		repo_path,
+		repo_path.gitpath(),
 		RepositoryOpenFlags::empty(),
 		Vec::<&Path>::new(),
 	)
 	.is_ok()
 }
 
-/// checks if the git repo at path `repo_path` is a bare repo
-pub fn is_bare_repo(repo_path: &str) -> Result<bool> {
-	let repo = Repository::open_ext(
-		repo_path,
-		RepositoryOpenFlags::empty(),
-		Vec::<&Path>::new(),
-	)?;
-
-	Ok(repo.is_bare())
-}
-
-///
-pub(crate) fn repo(repo_path: &str) -> Result<Repository> {
-	let repo = Repository::open_ext(
-		repo_path,
-		RepositoryOpenFlags::empty(),
-		Vec::<&Path>::new(),
-	)?;
-
-	if repo.is_bare() {
-		return Err(Error::Generic("bare repo".to_string()));
-	}
-
-	Ok(repo)
-}
-
 ///
 pub(crate) fn work_dir(repo: &Repository) -> Result<&Path> {
 	repo.workdir().ok_or(Error::NoWorkDir)
 }
 
 /// path to .git folder
-pub fn repo_dir(repo_path: &str) -> Result<PathBuf> {
+pub fn repo_dir(repo_path: &RepoPath) -> Result<PathBuf> {
 	let repo = repo(repo_path)?;
 	Ok(repo.path().to_owned())
 }
 
 ///
-pub fn repo_work_dir(repo_path: &str) -> Result<String> {
+pub fn repo_work_dir(repo_path: &RepoPath) -> Result<String> {
 	let repo = repo(repo_path)?;
 	work_dir(&repo)?.to_str().map_or_else(
 		|| Err(Error::Generic("invalid workdir".to_string())),
@@ -79,13 +55,13 @@ pub fn repo_work_dir(repo_path: &str) -> Result<String> {
 }
 
 ///
-pub fn get_head(repo_path: &str) -> Result<CommitId> {
+pub fn get_head(repo_path: &RepoPath) -> Result<CommitId> {
 	let repo = repo(repo_path)?;
 	get_head_repo(&repo)
 }
 
 ///
-pub fn get_head_tuple(repo_path: &str) -> Result<Head> {
+pub fn get_head_tuple(repo_path: &RepoPath) -> Result<Head> {
 	let repo = repo(repo_path)?;
 	let id = get_head_repo(&repo)?;
 	let name = get_head_refname(&repo)?;
@@ -111,7 +87,10 @@ pub fn get_head_repo(repo: &Repository) -> Result<CommitId> {
 }
 
 /// add a file diff from workingdir to stage (will not add removed files see `stage_addremoved`)
-pub fn stage_add_file(repo_path: &str, path: &Path) -> Result<()> {
+pub fn stage_add_file(
+	repo_path: &RepoPath,
+	path: &Path,
+) -> Result<()> {
 	scope_time!("stage_add_file");
 
 	let repo = repo(repo_path)?;
@@ -125,23 +104,31 @@ pub fn stage_add_file(repo_path: &str, path: &Path) -> Result<()> {
 }
 
 /// like `stage_add_file` but uses a pattern to match/glob multiple files/folders
-pub fn stage_add_all(repo_path: &str, pattern: &str) -> Result<()> {
+pub fn stage_add_all(
+	repo_path: &RepoPath,
+	pattern: &str,
+	stage_untracked: Option<ShowUntrackedFilesConfig>,
+) -> Result<()> {
 	scope_time!("stage_add_all");
 
 	let repo = repo(repo_path)?;
 
 	let mut index = repo.index()?;
 
-	let config = untracked_files_config_repo(&repo)?;
-
-	if config.include_none() {
-		index.update_all(vec![pattern], None)?;
+	let stage_untracked = if let Some(config) = stage_untracked {
+		config
 	} else {
+		untracked_files_config_repo(&repo)?
+	};
+
+	if stage_untracked.include_untracked() {
 		index.add_all(
 			vec![pattern],
 			IndexAddOption::DEFAULT,
 			None,
 		)?;
+	} else {
+		index.update_all(vec![pattern], None)?;
 	}
 
 	index.write()?;
@@ -150,7 +137,7 @@ pub fn stage_add_all(repo_path: &str, pattern: &str) -> Result<()> {
 }
 
 /// Undo last commit in repo
-pub fn undo_last_commit(repo_path: &str) -> Result<()> {
+pub fn undo_last_commit(repo_path: &RepoPath) -> Result<()> {
 	let repo = repo(repo_path)?;
 	let previous_commit = repo.revparse_single("HEAD~")?;
 
@@ -165,7 +152,10 @@ pub fn undo_last_commit(repo_path: &str) -> Result<()> {
 }
 
 /// stage a removed file
-pub fn stage_addremoved(repo_path: &str, path: &Path) -> Result<()> {
+pub fn stage_addremoved(
+	repo_path: &RepoPath,
+	path: &Path,
+) -> Result<()> {
 	scope_time!("stage_addremoved");
 
 	let repo = repo(repo_path)?;
@@ -197,6 +187,17 @@ pub(crate) fn repo_write_file(
 	Ok(())
 }
 
+///
+pub fn read_file(path: &Path) -> Result<String> {
+	use std::io::Read;
+
+	let mut file = File::open(path)?;
+	let mut buffer = Vec::new();
+	file.read_to_end(&mut buffer)?;
+
+	Ok(String::from_utf8(buffer)?)
+}
+
 #[cfg(test)]
 pub(crate) fn repo_read_file(
 	repo: &Repository,
@@ -242,7 +243,7 @@ mod tests {
 		let repo_path = root.as_os_str().to_str().unwrap();
 
 		assert_eq!(
-			stage_add_file(repo_path, file_path).is_ok(),
+			stage_add_file(&repo_path.into(), file_path).is_ok(),
 			false
 		);
 	}
@@ -252,7 +253,8 @@ mod tests {
 		let file_path = Path::new("file1.txt");
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		File::create(&root.join(file_path))
 			.unwrap()
@@ -275,7 +277,8 @@ mod tests {
 	fn test_staging_folder() -> Result<()> {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		let status_count = |s: StatusType| -> usize {
 			get_status(repo_path, s, None).unwrap().len()
@@ -291,7 +294,7 @@ mod tests {
 
 		assert_eq!(status_count(StatusType::WorkingDir), 3);
 
-		stage_add_all(repo_path, "a/d").unwrap();
+		stage_add_all(repo_path, "a/d", None).unwrap();
 
 		assert_eq!(status_count(StatusType::WorkingDir), 1);
 		assert_eq!(status_count(StatusType::Stage), 2);
@@ -303,7 +306,8 @@ mod tests {
 	fn test_undo_commit_empty_repo() {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		// expect to fail
 		assert!(undo_last_commit(repo_path).is_err());
@@ -313,7 +317,8 @@ mod tests {
 	fn test_undo_commit() {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		// write commit file test.txt
 		let c1 =
@@ -338,7 +343,8 @@ mod tests {
 	fn test_not_staging_untracked_folder() -> Result<()> {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		fs::create_dir_all(&root.join("a/d"))?;
 		File::create(&root.join(Path::new("a/d/f1.txt")))?
@@ -354,7 +360,7 @@ mod tests {
 
 		assert_eq!(get_statuses(repo_path), (0, 0));
 
-		stage_add_all(repo_path, "*").unwrap();
+		stage_add_all(repo_path, "*", None).unwrap();
 
 		assert_eq!(get_statuses(repo_path), (0, 0));
 
@@ -366,7 +372,8 @@ mod tests {
 		let file_path = Path::new("file1.txt");
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		let status_count = |s: StatusType| -> usize {
 			get_status(repo_path, s, None).unwrap().len()
@@ -400,7 +407,8 @@ mod tests {
 	fn test_staging_sub_git_folder() -> Result<()> {
 		let (_td, repo) = repo_init().unwrap();
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		let status_count = |s: StatusType| -> usize {
 			get_status(repo_path, s, None).unwrap().len()
@@ -410,7 +418,10 @@ mod tests {
 
 		fs::create_dir_all(sub)?;
 
-		debug_cmd_print(sub.to_str().unwrap(), "git init subgit");
+		debug_cmd_print(
+			&sub.to_str().unwrap().into(),
+			"git init subgit",
+		);
 
 		File::create(sub.join("subgit/foo.txt"))
 			.unwrap()
@@ -420,7 +431,7 @@ mod tests {
 		assert_eq!(status_count(StatusType::WorkingDir), 1);
 
 		//expect to fail
-		assert!(stage_add_all(repo_path, "sub").is_err());
+		assert!(stage_add_all(repo_path, "sub", None).is_err());
 
 		Ok(())
 	}
@@ -429,7 +440,8 @@ mod tests {
 	fn test_head_empty() -> Result<()> {
 		let (_td, repo) = repo_init_empty()?;
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		assert_eq!(get_head(repo_path).is_ok(), false);
 
@@ -440,7 +452,8 @@ mod tests {
 	fn test_head() -> Result<()> {
 		let (_td, repo) = repo_init()?;
 		let root = repo.path().parent().unwrap();
-		let repo_path = root.as_os_str().to_str().unwrap();
+		let repo_path: &RepoPath =
+			&root.as_os_str().to_str().unwrap().into();
 
 		assert_eq!(get_head(repo_path).is_ok(), true);
 
